/*
** Copyright (C) 2007 Jesse Chappell <jesse@essej.net>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#ifndef __ac3jack_coreaudio_spdifwriter__
#define __ac3jack_coreaudio_spdifwriter__

#include <string>
#include "boost/thread/thread.hpp"
#include "boost/thread/condition.hpp"
#include <cstdio>


#include "types.hpp"
#include "spdif_writer.hpp"
#include "ringbuffer.hpp"

#include "coreaudio/CoreAudioOutput.hpp"

namespace AC3JACK
{


class CoreAudioSpdifWriter
	: public SpdifWriter, public COREAUDIO::DataSource
{
  public:

	CoreAudioSpdifWriter(unsigned int samplerate=48000,
			     const std::string & devstr = "",
			     size_t bufsize = 32768);
	virtual ~CoreAudioSpdifWriter();

	virtual void start();

	/**
	 *   queues data to be written to the spdif.
	 *   this method is realtime safe
	 */
	virtual size_t write(unsigned char * data, size_t length);
	
	virtual int get_devices(DeviceInfoList & retlist);

	virtual void set_device (std::string devstr);
	virtual std::string get_device () { return _devstr;}

	virtual void set_allow_pcm (bool val);
	virtual bool get_allow_pcm () const { 
		if (_ca_out) return _ca_out->get_allow_pcm();
		else return false;
	}

	/**
	 * inherited callback from data source
	 */
	virtual uint32_t get_output(unsigned char * outdata, uint32_t databytes);

	virtual uint32_t get_output(unsigned char * outdata, unsigned char * outdata2, uint32_t databytes);


  protected:

	virtual void run_work_thread() {} // do nothing

	OSStatus	Init(AudioDeviceID output);
	std::string _devstr;
	
	RingBuffer<unsigned char>    * _buffer;
	unsigned int                   _samplerate;

	COREAUDIO::CoreAudioOutput                * _ca_out;
};



	

}


#endif
